@@ -60,6 +60,8 @@ static char *cleanstr(char *path, const char sep) {
 }
 
 static char *cleanpath(char *path) {
+    if (path == NULL || strlen(path) == 0)
+        return path;
     if (STREQ(path, "/"))
         return path;
     return cleanstr(path, SEP);
@@ -1323,6 +1325,8 @@ static int run_args(int argc, char **argv) {
     }
     code = run_command(line);
     free(line);
+    if (code == CMD_RES_OK && auto_save)
+        code = run_command("save");
     return (code == CMD_RES_OK || code == CMD_RES_QUIT) ? 0 : -1;
 }
 
