@@ -86,6 +86,16 @@ static struct value *lns_counter(struct info *info, struct value *str) {
     return lns_make_prim(L_COUNTER, ref(info), NULL, ref(str->string));
 }
 
+/* V_REGEXP -> V_LENS -> V_LENS */
+static struct value *lns_square(struct info *info, struct value *rxp,
+                                struct value *lns) {
+    assert(rxp->tag == V_REGEXP);
+    assert(lns->tag == V_LENS);
+    int check = info->error->aug->flags & AUG_TYPE_CHECK;
+
+    return lns_make_square(ref(info), ref(rxp->regexp), ref(lns->lens), check);
+}
+
 static struct value *make_exn_lns_error(struct info *info,
                                         struct lns_error *err,
                                         const char *text) {
@@ -147,6 +157,19 @@ static struct value *make_pathx_exn(struct info *info, struct pathx *p) {
     return v;
 }
 
+static struct value *pathx_parse_glue(struct info *info, struct value *tree,
+                                      struct value *path, struct pathx **p) {
+    assert(path->tag == V_STRING);
+    assert(tree->tag == V_TREE);
+
+    if (pathx_parse(tree->origin, info->error, path->string->str, true,
+                    NULL, p) != PATHX_NOERROR) {
+        return make_pathx_exn(ref(info), *p);
+    } else {
+        return NULL;
+    }
+}
+
 /* V_LENS -> V_STRING -> V_TREE */
 static struct value *lens_get(struct info *info, struct value *l,
                               struct value *str) {
@@ -224,11 +247,9 @@ static struct value *tree_set_glue(struct info *info, struct value *path,
         fake = tree->origin->children;
     }
 
-    if (pathx_parse(tree->origin, NULL, path->string->str, true, NULL, &p)
-        != PATHX_NOERROR) {
-        result = make_pathx_exn(ref(info), p);
+    result = pathx_parse_glue(info, tree, path, &p);
+    if (result != NULL)
         goto done;
-    }
 
     if (tree_set(p, val->string->str) == NULL) {
         result = make_exn_value(ref(info),
@@ -265,11 +286,9 @@ static struct value *tree_clear_glue(struct info *info, struct value *path,
         fake = tree->origin->children;
     }
 
-    if (pathx_parse(tree->origin, NULL, path->string->str, true, NULL, &p)
-        != PATHX_NOERROR) {
-        result = make_pathx_exn(ref(info), p);
+    result = pathx_parse_glue(info, tree, path, &p);
+    if (result != NULL)
         goto done;
-    }
 
     if (tree_set(p, NULL) == NULL) {
         result = make_exn_value(ref(info),
@@ -302,11 +321,9 @@ static struct value *tree_insert_glue(struct info *info, struct value *label,
     struct pathx *p = NULL;
     struct value *result = NULL;
 
-    if (pathx_parse(tree->origin, NULL, path->string->str, true, NULL, &p)
-        != PATHX_NOERROR) {
-        result = make_pathx_exn(ref(info), p);
+    result = pathx_parse_glue(info, tree, path, &p);
+    if (result != NULL)
         goto done;
-    }
 
     r = tree_insert(p, label->string->str, before);
     if (r != 0) {
@@ -349,11 +366,9 @@ static struct value *tree_rm_glue(struct info *info,
     struct pathx *p = NULL;
     struct value *result = NULL;
 
-    if (pathx_parse(tree->origin, NULL, path->string->str, true, NULL, &p)
-        != PATHX_NOERROR) {
-        result = make_pathx_exn(ref(info), p);
+    result = pathx_parse_glue(info, tree, path, &p);
+    if (result != NULL)
         goto done;
-    }
 
     if (tree_rm(p) == -1) {
         result = make_exn_value(ref(info), "Tree rm of %s failed",
@@ -536,6 +551,7 @@ struct module *builtin_init(struct error *error) {
     DEFINE_NATIVE(modl, "label",   1, lns_label, T_STRING, T_LENS);
     DEFINE_NATIVE(modl, "seq",     1, lns_seq, T_STRING, T_LENS);
     DEFINE_NATIVE(modl, "counter", 1, lns_counter, T_STRING, T_LENS);
+    DEFINE_NATIVE(modl, "square",  2, lns_square, T_REGEXP, T_LENS, T_LENS);
     /* Applying lenses (mostly for tests) */
     DEFINE_NATIVE(modl, "get",     2, lens_get, T_LENS, T_STRING, T_TREE);
     DEFINE_NATIVE(modl, "put",     3, lens_put, T_LENS, T_TREE, T_STRING,
