@@ -45,10 +45,12 @@ struct seq {
 
 struct state {
     struct info      *info;
+    struct span      *span;
     const char       *text;
     struct seq       *seqs;
     char             *key;
     char             *value;     /* GET_STORE leaves a value here */
+    char             *square;    /* last L_DEL from L_SQUARE */
     struct lns_error *error;
     /* We use the registers from a regular expression match to keep track
      * of the substring we are currently looking at. REGS are the registers
@@ -71,12 +73,14 @@ struct state {
 struct frame {
     struct lens     *lens;
     char            *key;
+    char            *square;
+    struct span     *span;
     union {
-        struct {
+        struct { /* MGET */
             char        *value;
             struct tree *tree;
         };
-        struct {
+        struct { /* M_PARSE */
             struct skel *skel;
             struct dict *dict;
         };
@@ -156,7 +160,8 @@ static struct skel *make_skel(struct lens *lens) {
 void free_skel(struct skel *skel) {
     if (skel == NULL)
         return;
-    if (skel->tag == L_CONCAT || skel->tag == L_STAR || skel->tag == L_MAYBE) {
+    if (skel->tag == L_CONCAT || skel->tag == L_STAR || skel->tag == L_MAYBE ||
+        skel->tag == L_SQUARE) {
         while (skel->skels != NULL) {
             struct skel *del = skel->skels;
             skel->skels = del->next;
@@ -383,6 +388,10 @@ static struct tree *get_del(struct lens *lens, struct state *state) {
         get_error(state, lens, "no match for del /%s/", pat);
         free(pat);
     }
+    if (lens->string == NULL) {
+        state->square = token(state);
+    }
+    update_span(state->span, REG_START(state), REG_END(state));
     return NULL;
 }
 
@@ -408,8 +417,14 @@ static struct tree *get_store(struct lens *lens, struct state *state) {
         get_error(state, lens, "More than one store in a subtree");
     else if (! REG_MATCHED(state))
         no_match_error(state, lens);
-    else
+    else {
         state->value = token(state);
+        if (state->span) {
+            state->span->value_start = REG_START(state);
+            state->span->value_end = REG_END(state);
+            update_span(state->span, REG_START(state), REG_END(state));
+        }
+    }
     return tree;
 }
 
@@ -435,8 +450,14 @@ static struct tree *get_key(struct lens *lens, struct state *state) {
     ensure0(lens->tag == L_KEY, state->info);
     if (! REG_MATCHED(state))
         no_match_error(state, lens);
-    else
+    else {
         state->key = token(state);
+        if (state->span) {
+            state->span->label_start = REG_START(state);
+            state->span->label_end = REG_END(state);
+            update_span(state->span, REG_START(state), REG_END(state));
+        }
+    }
     return NULL;
 }
 
@@ -656,17 +677,32 @@ static struct skel *parse_quant_maybe(struct lens *lens, struct state *state,
 static struct tree *get_subtree(struct lens *lens, struct state *state) {
     char *key = state->key;
     char *value = state->value;
+    struct span *span = state->span;
+
     struct tree *tree = NULL, *children;
 
     state->key = NULL;
     state->value = NULL;
+    if (state->info->flags & AUG_ENABLE_SPAN) {
+        state->span = make_span(state->info);
+        ERR_NOMEM(state->span == NULL, state->info);
+    }
+
     children = get_lens(lens->child, state);
 
     tree = make_tree(state->key, state->value, NULL, children);
+    tree->span = state->span;
+
+    if (state->span != NULL) {
+        update_span(span, state->span->span_start, state->span->span_end);
+    }
 
     state->key = key;
     state->value = value;
+    state->span = span;
     return tree;
+ error:
+    return NULL;
 }
 
 static struct skel *parse_subtree(struct lens *lens, struct state *state,
@@ -682,6 +718,42 @@ static struct skel *parse_subtree(struct lens *lens, struct state *state,
     return make_skel(lens);
 }
 
+static struct tree *get_square(struct lens *lens, struct state *state) {
+    ensure0(lens->tag == L_SQUARE, state->info);
+
+    struct tree *tree = NULL;
+    char *key = NULL, *square = NULL;
+
+    // get the child lens
+    tree = get_concat(lens->child, state);
+
+    key = state->key;
+    square = state->square;
+    ensure0(key != NULL, state->info);
+    ensure0(square != NULL, state->info);
+
+    if (strcmp(key, square) != 0) {
+        get_error(state, lens, "%s \"%s\" %s \"%s\"",
+                "Parse error: mismatched key in square lens, expecting", key,
+                "but got", square);
+    }
+
+    FREE(state->square);
+    return tree;
+}
+
+static struct skel *parse_square(struct lens *lens, struct state *state,
+                                 struct dict **dict) {
+    ensure0(lens->tag == L_SQUARE, state->info);
+    struct skel *skel, *sk;
+
+    skel = parse_concat(lens->child, state, dict);
+    sk = make_skel(lens);
+    sk->skels = skel;
+
+    return sk;
+}
+
 /*
  * Helpers for recursive lenses
  */
@@ -691,7 +763,7 @@ static void print_frames(struct rec_state *state) {
     for (int j = state->fused - 1; j >=0; j--) {
         struct frame *f = state->frames + j;
         for (int i=0; i < state->lvl; i++) fputc(' ', stderr);
-        fprintf(stderr, "%2d %s %s", j, f->key, f->value);
+        fprintf(stderr, "%2d %s %s %s", j, f->key, f->value, f->square);
         if (f->tree == NULL) {
             fprintf(stderr, " - ");
         } else {
@@ -760,8 +832,10 @@ static void get_terminal(struct frame *top, struct lens *lens,
     top->tree = get_lens(lens, state);
     top->key = state->key;
     top->value = state->value;
+    top->square = state->square;
     state->key = NULL;
     state->value = NULL;
+    state->square = NULL;
 }
 
 static void parse_terminal(struct frame *top, struct lens *lens,
@@ -813,15 +887,24 @@ static void visit_enter(struct lens *lens,
         struct frame *f = push_frame(rec_state, lens);
         f->key = state->key;
         f->value = state->value;
+        f->span = state->span;
         state->key = NULL;
         state->value = NULL;
+    } else if (lens->tag == L_MAYBE) {
+        push_frame(rec_state, lens);
+        if (state->info->flags & AUG_ENABLE_SPAN) {
+            state->span = make_span(state->info);
+            ERR_NOMEM(state->span == NULL, state->info);
+        }
     }
+ error:
+    return;
 }
 
 static void get_combine(struct rec_state *rec_state,
                         struct lens *lens, uint n) {
     struct tree *tree = NULL, *tail = NULL;
-    char *key = NULL, *value = NULL;
+    char *key = NULL, *value = NULL, *square = NULL;
     struct frame *top = NULL;
 
     if (n > 0)
@@ -841,11 +924,16 @@ static void get_combine(struct rec_state *rec_state,
             ensure(value == NULL, rec_state->state->info);
             value = top->value;
         }
+        if (top->square != NULL) {
+            ensure(square == NULL, rec_state->state->info);
+            square = top->square;
+        }
     }
     top = push_frame(rec_state, lens);
     top->tree = tree;
     top->key = key;
     top->value = value;
+    top->square = square;
  error:
     return;
 }
@@ -899,12 +987,15 @@ static void visit_exit(struct lens *lens,
         struct frame *top = top_frame(rec_state);
         if (rec_state->mode == M_GET) {
             struct tree *tree;
+            // FIXME: tree may leak if pop_frame ensure0 fail
             tree = make_tree(top->key, top->value, NULL, top->tree);
+            tree->span = state->span;
             ERR_NOMEM(tree == NULL, lens->info);
             top = pop_frame(rec_state);
             ensure(lens == top->lens, state->info);
             state->key = top->key;
             state->value = top->value;
+            state->span = top->span;
             pop_frame(rec_state);
             top = push_frame(rec_state, lens);
             top->tree = tree;
@@ -948,11 +1039,39 @@ static void visit_exit(struct lens *lens,
         else
             parse_combine(rec_state, lens, n);
     } else if (lens->tag == L_MAYBE) {
-        uint n = (top_frame(rec_state)->lens == lens->child) ? 1 : 0;
+        uint n = 1;
+        if (rec_state->fused > 0
+            && top_frame(rec_state)->lens == lens->child) {
+            n = 2;
+        }
         if (rec_state->mode == M_GET)
             get_combine(rec_state, lens, n);
         else
             parse_combine(rec_state, lens, n);
+    } else if (lens->tag == L_SQUARE) {
+        if (rec_state->mode == M_GET) {
+            char *key, *square;
+
+            key = top_frame(rec_state)->key;
+            square = top_frame(rec_state)->square;
+
+            ensure(key != NULL, state->info);
+            ensure(square != NULL, state->info);
+
+            // raise syntax error if they are not equals
+            if (strcmp(key, square) != 0){
+                get_error(state, lens, "%s \"%s\" %s \"%s\"",
+                        "Parse error: mismatched key in square lens, expecting",
+                        key, "but got", square);
+                state->error->pos = end - strlen(square);
+                goto error;
+            }
+
+            FREE(square);
+            get_combine(rec_state, lens, 1);
+        } else {
+            parse_combine(rec_state, lens, 1);
+        }
     } else {
         top_frame(rec_state)->lens = lens;
     }
@@ -981,6 +1100,8 @@ static struct frame *rec_process(enum mode_t mode, struct lens *lens,
     int r;
     struct jmt_visitor visitor;
     struct rec_state rec_state;
+    int i;
+    struct frame *f = NULL;
 
     MEMZERO(&rec_state, 1);
     MEMZERO(&visitor, 1);
@@ -1028,6 +1149,19 @@ static struct frame *rec_process(enum mode_t mode, struct lens *lens,
     jmt_free_parse(visitor.parse);
     return rec_state.frames;
  error:
+
+    for(i = 0; i < rec_state.fused; i++) {
+        f = nth_frame(&rec_state, i);
+        FREE(f->key);
+        FREE(f->square);
+        if (mode == M_GET) {
+            FREE(f->value);
+            free_tree(f->tree);
+        } else if (mode == M_PARSE) {
+            free_skel(f->skel);
+            free_dict(f->dict);
+        }
+    }
     FREE(rec_state.frames);
     goto done;
 }
@@ -1101,6 +1235,9 @@ static struct tree *get_lens(struct lens *lens, struct state *state) {
     case L_MAYBE:
         tree = get_quant_maybe(lens, state);
         break;
+    case L_SQUARE:
+        tree = get_square(lens, state);
+        break;
     default:
         BUG_ON(true, state->info, "illegal lens tag %d", lens->tag);
         break;
@@ -1176,6 +1313,10 @@ struct tree *lns_get(struct info *info, struct lens *lens, const char *text,
         get_error(&state, lens, "get left unused value %s", state.value);
         free(state.value);
     }
+    if (state.square != NULL) {
+        get_error(&state, lens, "get left unused square %s", state.square);
+        free(state.square);
+    }
     if (partial && state.error == NULL) {
         get_error(&state, lens, "Get did not match entire input");
     }
@@ -1237,6 +1378,9 @@ static struct skel *parse_lens(struct lens *lens, struct state *state,
     case L_MAYBE:
         skel = parse_quant_maybe(lens, state, dict);
         break;
+    case L_SQUARE:
+        skel = parse_square(lens, state, dict);
+        break;
     default:
         BUG_ON(true, state->info, "illegal lens tag %d", lens->tag);
         break;
