@@ -23,6 +23,7 @@
 #include <config.h>
 #include "info.h"
 #include "internal.h"
+#include "memory.h"
 #include "ref.h"
 
 /*
@@ -113,6 +114,48 @@ void free_info(struct info *info) {
     free(info);
 }
 
+struct span *make_span(struct info *info) {
+    struct span *span = NULL;
+    if (ALLOC(span) < 0) {
+        return NULL;
+    }
+    /* UINT_MAX means span is not initialized yet */
+    span->span_start = UINT_MAX;
+    span->filename = ref(info->filename);
+    return span;
+}
+
+void free_span(struct span *span) {
+    if (span == NULL)
+        return;
+    unref(span->filename, string);
+    free(span);
+}
+
+void print_span(struct span *span) {
+    if (span == NULL)
+        return;
+    printf("%s label=(%i:%i) value=(%i:%i) span=(%i,%i)\n",
+            span->filename->str,
+            span->label_start, span->label_end,
+            span->value_start, span->value_end,
+            span->span_start, span->span_end);
+}
+
+void update_span(struct span *node_info, int x, int y) {
+    if (node_info == NULL)
+        return;
+    if (node_info->span_start == UINT_MAX) {
+        node_info->span_start = x;
+        node_info->span_end = y;
+    } else {
+        if (node_info->span_start > x)
+            node_info->span_start = x;
+        if (node_info->span_end < y)
+            node_info->span_end = y;
+    }
+}
+
 /*
  * Local variables:
  *  indent-tabs-mode: nil
