@@ -26,6 +26,7 @@
 #include "lens.h"
 #include "memory.h"
 #include "errcode.h"
+#include "internal.h"
 
 /* This enum must be kept in sync with type_offs and ntypes */
 enum lens_type {
@@ -53,12 +54,15 @@ static struct value *typecheck_iter(struct info *info, struct lens *l);
 static struct value *typecheck_maybe(struct info *info, struct lens *l);
 
 /* Lens names for pretty printing */
+/* keep order in sync with enum type */
 static const char *const tags[] = {
     "del", "store", "value", "key", "label", "seq", "counter",
     "concat", "union",
-    "subtree", "star", "maybe", "rec"
+    "subtree", "star", "maybe", "rec", "square"
 };
 
+#define ltag(lens) (tags[lens->tag - L_DEL])
+
 static const struct string digits_string = {
     .ref = REF_MAX, .str = (char *) "[0123456789]+"
 };
@@ -394,6 +398,65 @@ struct value *lns_make_maybe(struct info *info, struct lens *l, int check) {
     return make_lens_value(lens);
 }
 
+/* Build a square lens as
+ *   key REG . lns . del REG MATCHED
+ * where MATCHED is whatever the key lens matched (the inability to express
+ * this with other lenses makes the square primitve necessary
+ */
+struct value *lns_make_square(struct info *info,
+                              struct regexp *reg,
+                              struct lens *lns, int check) {
+    struct value *key = NULL, *del = NULL;
+    struct value *cnt1 = NULL, *cnt2 = NULL, *res = NULL;
+    struct lens *sqr = NULL;
+
+    res = lns_make_prim(L_KEY, ref(info), ref(reg), NULL);
+    if (EXN(res))
+        goto error;
+    key = res;
+
+    res = lns_make_prim(L_DEL, ref(info), ref(reg), NULL);
+    if (EXN(res))
+        goto error;
+    del = res;
+
+    // typechecking is handled when concatenating lenses
+    res = lns_make_concat(ref(info), ref(key->lens), ref(lns), check);
+    if (EXN(res))
+        goto error;
+    cnt1 = res;
+
+    res = lns_make_concat(ref(info), ref(cnt1->lens), ref(del->lens), check);
+    if (EXN(res))
+        goto error;
+    cnt2 = res;
+
+    sqr = make_lens_unop(L_SQUARE, ref(info), ref(cnt2->lens));
+    ERR_NOMEM(sqr == NULL, info);
+
+    for (int t=0; t < ntypes; t++)
+        ltype(sqr, t) = ref(ltype(cnt2->lens, t));
+    sqr->recursive = cnt2->lens->recursive;
+    sqr->rec_internal = cnt2->lens->rec_internal;
+    sqr->consumes_value = cnt2->lens->consumes_value;
+
+    res = make_lens_value(sqr);
+    ERR_NOMEM(res == NULL, info);
+    sqr = NULL;
+
+ error:
+    unref(info, info);
+    unref(reg, regexp);
+    unref(lns, lens);
+
+    unref(key, value);
+    unref(del, value);
+    unref(cnt1, value);
+    unref(cnt2, value);
+    unref(sqr, lens);
+    return res;
+}
+
 /*
  * Lens primitives
  */
@@ -474,7 +537,7 @@ struct value *lns_make_prim(enum lens_tag tag, struct info *info,
                                  string->str);
             goto error;
         }
-    } else if (tag == L_DEL) {
+    } else if (tag == L_DEL && string != NULL) {
         int cnt;
         const char *dflt = string->str;
         cnt = regexp_match(regexp, dflt, strlen(dflt), 0, NULL);
@@ -794,6 +857,8 @@ void free_lens(struct lens *lens) {
         return;
     ensure(lens->ref == 0, lens->info);
 
+    if (debugging("lenses"))
+        dump_lens_tree(lens);
     switch (lens->tag) {
     case L_DEL:
         unref(lens->regexp, regexp);
@@ -812,6 +877,7 @@ void free_lens(struct lens *lens) {
     case L_SUBTREE:
     case L_STAR:
     case L_MAYBE:
+    case L_SQUARE:
         unref(lens->child, lens);
         break;
     case L_CONCAT:
@@ -1116,6 +1182,9 @@ int lns_format_atype(struct lens *l, char **buf) {
     case L_REC:
         return lns_format_rec_atype(l, buf);
         break;
+    case L_SQUARE:
+        return lns_format_concat_atype(l->child, buf);
+        break;
     default:
         BUG_LENS_TAG(l);
         break;
@@ -1802,6 +1871,10 @@ static void propagate_type(struct lens *l, enum lens_type lt) {
     case L_REC:
         /* Nothing to do */
         break;
+    case L_SQUARE:
+        propagate_type(l->child, lt);
+        ltype(l, lt) = ref(ltype(l->child, lt));
+        break;
     default:
         BUG_LENS_TAG(l);
         break;
@@ -1878,6 +1951,7 @@ static struct value *typecheck(struct lens *l, int check) {
         exn = typecheck_n(l, lns_make_union, check);
         break;
     case L_SUBTREE:
+    case L_SQUARE:
         exn = typecheck(l->child, check);
         break;
     case L_STAR:
@@ -1992,6 +2066,7 @@ static int ctype_nullable(struct lens *lens, struct value **exn) {
         }
         break;
     case L_SUBTREE:
+    case L_SQUARE:
         ret = ctype_nullable(lens->child, exn);
         nullable = lens->child->ctype_nullable;
         break;
@@ -2023,9 +2098,15 @@ struct value *lns_check_rec(struct info *info,
     struct value *result = NULL;
 
     ensure(rec->tag == L_REC, info);
-    ensure(body->recursive, info);
     ensure(rec->rec_internal, info);
 
+    /* The user might have written down a regular lens with 'let rec' */
+    if (! body->recursive) {
+        result = make_lens_value(ref(body));
+        ERR_NOMEM(result == NULL, info);
+        return result;
+    }
+
     /* To help memory management, we avoid the cycle inherent ina recursive
      * lens by using two instances of an L_REC lens. One is marked with
      * rec_internal, and used inside the body of the lens. The other is the
@@ -2086,6 +2167,100 @@ struct value *lns_check_rec(struct info *info,
     return result;
 }
 
+#if ENABLE_DEBUG
+void dump_lens_tree(struct lens *lens){
+    static int count = 0;
+    FILE *fp;
+
+    fp = debug_fopen("lens_%02d_%s.dot", count++, ltag(lens));
+    if (fp == NULL)
+        return;
+
+    fprintf(fp, "digraph \"%s\" {\n", "lens");
+    dump_lens(fp, lens);
+    fprintf(fp, "}\n");
+
+    fclose(fp);
+}
+
+void dump_lens(FILE *out, struct lens *lens){
+    int i = 0;
+    struct regexp *re;
+
+    fprintf(out, "\"%p\" [ shape = box, label = \"%s\\n", lens, ltag(lens));
+
+    for (int t=0; t < ntypes; t++) {
+        re = ltype(lens, t);
+        if (re == NULL)
+            continue;
+        fprintf(out, "%s=",lens_type_names[t]);
+        print_regexp(out, re);
+        fprintf(out, "\\n");
+    }
+
+    fprintf(out, "recursive=%x\\n", lens->recursive);
+    fprintf(out, "rec_internal=%x\\n", lens->rec_internal);
+    fprintf(out, "consumes_value=%x\\n", lens->consumes_value);
+    fprintf(out, "ctype_nullable=%x\\n", lens->ctype_nullable);
+    fprintf(out, "\"];\n");
+    switch(lens->tag){
+    case L_DEL:
+        break;
+    case L_STORE:
+        break;
+    case L_VALUE:
+        break;
+    case L_KEY:
+        break;
+    case L_LABEL:
+        break;
+    case L_SEQ:
+        break;
+    case L_COUNTER:
+        break;
+    case L_CONCAT:
+        for(i = 0; i<lens->nchildren;i++){
+            fprintf(out, "\"%p\" -> \"%p\"\n", lens, lens->children[i]);
+            dump_lens(out, lens->children[i]);
+        }
+        break;
+    case L_UNION:
+        for(i = 0; i<lens->nchildren;i++){
+            fprintf(out, "\"%p\" -> \"%p\"\n", lens, lens->children[i]);
+            dump_lens(out, lens->children[i]);
+        }
+        break;
+    case L_SUBTREE:
+        fprintf(out, "\"%p\" -> \"%p\"\n", lens, lens->child);
+        dump_lens(out, lens->child);
+        break;
+    case L_STAR:
+        fprintf(out, "\"%p\" -> \"%p\"\n", lens, lens->child);
+        dump_lens(out, lens->child);
+
+        break;
+    case L_MAYBE:
+        fprintf(out, "\"%p\" -> \"%p\"\n", lens, lens->child);
+        dump_lens(out, lens->child);
+
+        break;
+    case L_REC:
+        if (lens->rec_internal == 0){
+            fprintf(out, "\"%p\" -> \"%p\"\n", lens, lens->child);
+            dump_lens(out, lens->body);
+        }
+        break;
+    case L_SQUARE:
+        fprintf(out, "\"%p\" -> \"%p\"\n", lens, lens->child);
+        dump_lens(out, lens->child);
+        break;
+    default:
+        fprintf(out, "ERROR\n");
+        break;
+    }
+}
+#endif
+
 /*
  * Local variables:
  *  indent-tabs-mode: nil
