@@ -27,6 +27,7 @@
 #include "fa.h"
 #include "jmt.h"
 
+/* keep in sync with tag name table */
 enum lens_tag {
     L_DEL = 42,    /* Shift tag values so we fail fast(er) on bad pointers */
     L_STORE,
@@ -40,7 +41,8 @@ enum lens_tag {
     L_SUBTREE,
     L_STAR,
     L_MAYBE,
-    L_REC
+    L_REC,
+    L_SQUARE
 };
 
 /* A lens. The way the type information is computed is a little
@@ -77,7 +79,6 @@ struct lens {
     struct regexp            *atype;
     struct regexp            *ktype;
     struct regexp            *vtype;
-    struct jmt               *jmt;    /* NULL when recursive == 0 */
     unsigned int              value : 1;
     unsigned int              key : 1;
     unsigned int              recursive : 1;
@@ -88,11 +89,14 @@ struct lens {
     union {
         /* Primitive lenses */
         struct {                   /* L_DEL uses both */
+            /* L_DEL string set to NULL means it belongs to parent L_SQUARE lens
+             * and the put and create copy the current key
+             */
             struct regexp *regexp; /* L_STORE, L_KEY */
             struct string *string; /* L_VALUE, L_LABEL, L_SEQ, L_COUNTER */
         };
         /* Combinators */
-        struct lens *child;         /* L_SUBTREE, L_STAR, L_MAYBE */
+        struct lens *child;         /* L_SUBTREE, L_STAR, L_MAYBE, L_SQUARE */
         struct {                    /* L_UNION, L_CONCAT */
             unsigned int nchildren;
             struct lens **children;
@@ -139,6 +143,8 @@ struct value *lns_make_plus(struct info *, struct lens *,
                             int check);
 struct value *lns_make_maybe(struct info *, struct lens *,
                              int check);
+struct value *lns_make_square(struct info *, struct regexp *, struct lens *,
+                              int check);
 
 /* Pretty-print a lens */
 char *format_lens(struct lens *l);
@@ -240,6 +246,11 @@ void free_lens(struct lens *lens);
  */
 char *enc_format(const char *e, size_t len);
 
+#if ENABLE_DEBUG
+void dump_lens_tree(struct lens *lens);
+void dump_lens(FILE *out, struct lens *lens);
+#endif
+
 #endif
 
 
