@@ -49,6 +49,7 @@ static const int glob_flags = GLOB_NOSORT;
 /* Loaded files are tracked underneath METATREE. When a file with name
  * FNAME is loaded, certain entries are made under METATREE / FNAME:
  *   path      : path where tree for FNAME is put
+ *   mtime     : time of last modification of the file as reported by stat(2)
  *   lens/info : information about where the applied lens was loaded from
  *   lens/id   : unique hexadecimal id of the lens
  *   error     : indication of errors during processing FNAME, or NULL
@@ -60,6 +61,7 @@ static const int glob_flags = GLOB_NOSORT;
 static const char *const s_path = "path";
 static const char *const s_lens = "lens";
 static const char *const s_info = "info";
+static const char *const s_mtime = "mtime";
 
 static const char *const s_error = "error";
 /* These are all put underneath "error" */
@@ -111,6 +113,66 @@ static bool is_regular_file(const char *path) {
     return S_ISREG(st.st_mode);
 }
 
+static char *mtime_as_string(struct augeas *aug, const char *fname) {
+    int r;
+    struct stat st;
+    char *result = NULL;
+
+    if (fname == NULL) {
+        result = strdup("0");
+        ERR_NOMEM(result == NULL, aug);
+        goto done;
+    }
+
+    r = stat(fname, &st);
+    if (r < 0) {
+        /* If we fail to stat, silently ignore the error
+         * and report an impossible mtime */
+        result = strdup("0");
+        ERR_NOMEM(result == NULL, aug);
+    } else {
+        r = xasprintf(&result, "%ld", (long) st.st_mtime);
+        ERR_NOMEM(r < 0, aug);
+    }
+ done:
+    return result;
+ error:
+    FREE(result);
+    return NULL;
+}
+
+static bool file_current(struct augeas *aug, const char *fname,
+                         struct tree *finfo) {
+    struct tree *mtime = tree_child(finfo, s_mtime);
+    struct tree *file = NULL, *path = NULL;
+    int r;
+    struct stat st;
+    int64_t mtime_i;
+
+    if (mtime == NULL || mtime->value == NULL)
+        return false;
+
+    r = xstrtoint64(mtime->value, 10, &mtime_i);
+    if (r < 0) {
+        /* Ignore silently and err on the side of caution */
+        return false;
+    }
+
+    r = stat(fname, &st);
+    if (r < 0)
+        return false;
+
+    if (mtime_i != (int64_t) st.st_mtime)
+        return false;
+
+    path = tree_child(finfo, s_path);
+    if (path == NULL)
+        return false;
+
+    file = tree_find(aug, path->value);
+    return (file != NULL && ! file->dirty);
+}
+
 static int filter_generate(struct tree *xfm, const char *root,
                            int *nmatches, char ***matches) {
     glob_t globbuf;
@@ -234,30 +296,29 @@ static char *err_path(const char *filename) {
 }
 
 ATTRIBUTE_FORMAT(printf, 4, 5)
-static int err_set(struct augeas *aug, char **ep, const char *sub,
-                   const char *format, ...) {
+static void err_set(struct augeas *aug,
+                    struct tree *err_info, const char *sub,
+                    const char *format, ...) {
     int r;
     va_list ap;
-    size_t ep_len = strlen(*ep);
     char *value = NULL;
-
-    r = pathjoin(ep, 1, sub);
-    if (r < 0)
-        goto error;
+    struct tree *tree = NULL;
 
     va_start(ap, format);
     r = vasprintf(&value, format, ap);
     va_end(ap);
-    if (r < 0) {
+    if (r < 0)
         value = NULL;
-        goto error;
-    }
+    ERR_NOMEM(r < 0, aug);
+
+    tree = tree_child_cr(err_info, sub);
+    ERR_NOMEM(tree == NULL, aug);
+
+    r = tree_set_value(tree, value);
+    ERR_NOMEM(r < 0, aug);
 
-    r = aug_set(aug, *ep, value);
  error:
     free(value);
-    (*ep)[ep_len] = '\0';
-    return (r < 0) ? -1 : 0;
 }
 
 /* Record an error in the tree. The error will show up underneath
@@ -269,51 +330,65 @@ static int store_error(struct augeas *aug,
                        const char *filename, const char *path,
                        const char *status, int errnum,
                        const struct lns_error *err, const char *text) {
-    char *ep = err_path(filename);
+    struct tree *err_info = NULL, *finfo = NULL;
+    char *fip = NULL;
     int r;
     int result = -1;
 
-    if (ep == NULL)
-        return -1;
+    r = pathjoin(&fip, 2, AUGEAS_META_FILES, filename);
+    ERR_NOMEM(r < 0, aug);
+
+    finfo = tree_find_cr(aug, fip);
+    ERR_BAIL(aug);
 
-    aug_rm(aug, ep);
     if (status != NULL) {
-        r = aug_set(aug, ep, status);
-        if (r < 0)
-            goto done;
+        err_info = tree_child_cr(finfo, s_error);
+        ERR_NOMEM(err_info == NULL, aug);
+
+        r = tree_set_value(err_info, status);
+        ERR_NOMEM(r < 0, aug);
 
         /* Errors from err_set are ignored on purpose. We try
          * to report as much as we can */
         if (err != NULL) {
             if (err->pos >= 0) {
                 size_t line, ofs;
-                err_set(aug, &ep, s_pos, "%d", err->pos);
+                err_set(aug, err_info, s_pos, "%d", err->pos);
                 if (text != NULL) {
                     calc_line_ofs(text, err->pos, &line, &ofs);
-                    err_set(aug, &ep, s_line, "%zd", line);
-                    err_set(aug, &ep, s_char, "%zd", ofs);
+                    err_set(aug, err_info, s_line, "%zd", line);
+                    err_set(aug, err_info, s_char, "%zd", ofs);
                 }
             }
             if (err->path != NULL) {
-                err_set(aug, &ep, s_path, "%s%s", path, err->path);
+                err_set(aug, err_info, s_path, "%s%s", path, err->path);
             }
             if (err->lens != NULL) {
                 char *fi = format_info(err->lens->info);
                 if (fi != NULL) {
-                    err_set(aug, &ep, s_lens, "%s", fi);
+                    err_set(aug, err_info, s_lens, "%s", fi);
                     free(fi);
                 }
             }
-            err_set(aug, &ep, s_message, "%s", err->message);
+            err_set(aug, err_info, s_message, "%s", err->message);
         } else if (errnum != 0) {
             const char *msg = strerror(errnum);
-            err_set(aug, &ep, s_message, "%s", msg);
+            err_set(aug, err_info, s_message, "%s", msg);
+        }
+    } else {
+        /* No error, nuke the error node if it exists */
+        err_info = tree_child(finfo, s_error);
+        if (err_info != NULL) {
+            tree_unlink_children(aug, err_info);
+            pathx_symtab_remove_descendants(aug->symtab, err_info);
+            tree_unlink(err_info);
         }
     }
 
+    tree_clean(finfo);
     result = 0;
- done:
-    free(ep);
+ error:
+    free(fip);
     return result;
 }
 
@@ -326,7 +401,8 @@ static int store_error(struct augeas *aug,
  * Returns 0 on success, -1 on error
  */
 static int add_file_info(struct augeas *aug, const char *node,
-                         struct lens *lens, const char *lens_name) {
+                         struct lens *lens, const char *lens_name,
+                         const char *filename) {
     struct tree *file, *tree;
     char *tmp = NULL;
     int r;
@@ -348,6 +424,13 @@ static int add_file_info(struct augeas *aug, const char *node,
     r = tree_set_value(tree, node);
     ERR_NOMEM(r < 0, aug);
 
+    /* Set 'mtime' */
+    tmp = mtime_as_string(aug, filename);
+    ERR_BAIL(aug);
+    tree = tree_child_cr(file, s_mtime);
+    ERR_NOMEM(tree == NULL, aug);
+    tree_store_value(tree, &tmp);
+
     /* Set 'lens/info' */
     tmp = format_info(lens->info);
     ERR_NOMEM(tmp == NULL, aug);
@@ -362,6 +445,8 @@ static int add_file_info(struct augeas *aug, const char *node,
     r = tree_set_value(tree, lens_name);
     ERR_NOMEM(r < 0, aug);
 
+    tree_clean(file);
+
     result = 0;
  error:
     free(path);
@@ -399,12 +484,13 @@ static int load_file(struct augeas *aug, struct lens *lens,
     struct tree *tree = NULL;
     char *path = NULL;
     struct lns_error *err = NULL;
-    int result = -1, r;
+    struct span *span = NULL;
+    int result = -1, r, text_len = 0;
 
     path = file_name_path(aug, filename);
     ERR_NOMEM(path == NULL, aug);
 
-    r = add_file_info(aug, path, lens, lens_name);
+    r = add_file_info(aug, path, lens, lens_name, filename);
     if (r < 0)
         goto done;
 
@@ -413,18 +499,24 @@ static int load_file(struct augeas *aug, struct lens *lens,
         err_status = "read_failed";
         goto done;
     }
-    text = append_newline(text, strlen(text));
+    text_len = strlen(text);
+    text = append_newline(text, text_len);
 
     struct info *info;
     make_ref(info);
     make_ref(info->filename);
-    info->filename->str = filename;
+    info->filename->str = strdup(filename);
     info->error = aug->error;
+    info->flags = aug->flags;
     info->first_line = 1;
 
+    if (aug->flags & AUG_ENABLE_SPAN) {
+        span = make_span(info);
+        ERR_NOMEM(span == NULL, info);
+    }
+
     tree = lns_get(info, lens, text, &err);
 
-    info->filename->str = NULL;
     unref(info, info);
 
     if (err != NULL) {
@@ -432,7 +524,15 @@ static int load_file(struct augeas *aug, struct lens *lens,
         goto done;
     }
 
-    tree_replace(aug->origin, path, tree);
+    tree_replace(aug, path, tree);
+
+    /* top level node span entire file length */
+    if (span != NULL) {
+        tree->parent->span = span;
+        tree->parent->span->span_start = 0;
+        tree->parent->span->span_end = text_len;
+    }
+
     tree = NULL;
 
     result = 0;
@@ -504,11 +604,10 @@ static struct lens *xfm_lens(struct augeas *aug,
 static void xfm_error(struct tree *xfm, const char *msg) {
     char *v = strdup(msg);
     char *l = strdup("error");
-    struct tree *e = NULL;
 
     if (l == NULL || v == NULL)
         return;
-    e = tree_append(xfm, l, v);
+    tree_append(xfm, l, v);
 }
 
 int transform_validate(struct augeas *aug, struct tree *xfm) {
@@ -602,7 +701,8 @@ int transform_load(struct augeas *aug, struct tree *xfm) {
     for (int i=0; i < nmatches; i++) {
         const char *filename = matches[i] + strlen(aug->root) - 1;
         struct tree *finfo = file_info(aug, filename);
-        if (finfo != NULL && tree_child(finfo, s_lens) != NULL) {
+        if (finfo != NULL && !finfo->dirty &&
+            tree_child(finfo, s_lens) != NULL) {
             const char *s = xfm_lens_name(finfo);
             char *fpath = file_name_path(aug, matches[i]);
             transform_file_error(aug, "mxfm_load", filename,
@@ -610,9 +710,11 @@ int transform_load(struct augeas *aug, struct tree *xfm) {
                                  s, lens_name);
             aug_rm(aug, fpath);
             free(fpath);
-        } else {
+        } else if (!file_current(aug, matches[i], finfo)) {
             load_file(aug, lens, lens_name, matches[i]);
         }
+        if (finfo != NULL)
+            finfo->dirty = 0;
         FREE(matches[i]);
     }
     lens_release(lens);
@@ -749,11 +851,9 @@ static int file_saved_event(struct augeas *aug, const char *path) {
     struct tree *dummy;
     int r;
 
-    r = pathx_parse(aug->origin, err_of_aug(aug),
-                    AUGEAS_EVENTS_SAVED "[last()]",
-                    true, NULL, &px);
-    if (r != PATHX_NOERROR)
-        return -1;
+    px = pathx_aug_parse(aug, aug->origin, AUGEAS_EVENTS_SAVED "[last()]",
+                         true);
+    ERR_BAIL(aug);
 
     if (pathx_find_one(px, &dummy) == 1) {
         r = tree_insert(px, saved, 0);
@@ -945,7 +1045,7 @@ int transform_save(struct augeas *aug, struct tree *xfm,
     result = 1;
 
  done:
-    r = add_file_info(aug, path, lens, lens_name);
+    r = add_file_info(aug, path, lens, lens_name, augorig);
     if (r < 0) {
         err_status = "file_info";
         result = -1;
