@@ -40,26 +40,6 @@ struct augeas *aug = NULL;
         exit(EXIT_FAILURE);                                         \
     } while(0)
 
-static void run(CuTest *tc, const char *format, ...) {
-    char *command;
-    va_list args;
-    int r;
-
-    va_start(args, format);
-    r = vasprintf(&command, format, args);
-    va_end (args);
-    if (r < 0)
-        CuFail(tc, "Failed to format command (out of memory)");
-    r = system(command);
-    if (r < 0 || (WIFEXITED(r) && WEXITSTATUS(r) != 0)) {
-        char *msg;
-        r = asprintf(&msg, "Command %s failed with status %d\n",
-                     command, WEXITSTATUS(r));
-        CuFail(tc, msg);
-        free(msg);
-    }
-}
-
 static void setup(CuTest *tc) {
     char *lensdir;
 
@@ -132,7 +112,7 @@ static void testMultipleXfm(CuTest *tc) {
     r = aug_set(aug, "/augeas/load/Yum2/incl", "/etc/yum.repos.d/*");
     CuAssertIntEquals(tc, 0, r);
 
-    r = aug_set(aug, "/files/etc/yum.repos.d/fedora.repo/fedora/enabled", "1");
+    r = aug_set(aug, "/files/etc/yum.repos.d/fedora.repo/fedora/enabled", "0");
     CuAssertIntEquals(tc, 0, r);
 
     r = aug_save(aug);
@@ -141,6 +121,30 @@ static void testMultipleXfm(CuTest *tc) {
     CuAssertIntEquals(tc, AUG_EMXFM, r);
 }
 
+static void testMtime(CuTest *tc) {
+    const char *s, *mtime2;
+    char *mtime1;
+    int r;
+
+    r = aug_set(aug, "/files/etc/hosts/1/alias[last() + 1]", "new");
+    CuAssertIntEquals(tc, 0, r);
+
+    r = aug_get(aug, "/augeas/files/etc/hosts/mtime", &s);
+    CuAssertIntEquals(tc, 1, r);
+    mtime1 = strdup(s);
+    CuAssertPtrNotNull(tc, mtime1);
+
+
+    r = aug_save(aug);
+    CuAssertIntEquals(tc, 0, r);
+
+    r = aug_get(aug, "/augeas/files/etc/hosts/mtime", &mtime2);
+    CuAssertIntEquals(tc, 1, r);
+
+    CuAssertStrNotEqual(tc, mtime1, mtime2);
+    CuAssertStrNotEqual(tc, "0", mtime2);
+}
+
 int main(void) {
     char *output = NULL;
     CuSuite* suite = CuSuiteNew();
@@ -162,6 +166,7 @@ int main(void) {
     SUITE_ADD_TEST(suite, testSaveNewFile);
     SUITE_ADD_TEST(suite, testNonExistentLens);
     SUITE_ADD_TEST(suite, testMultipleXfm);
+    SUITE_ADD_TEST(suite, testMtime);
 
     CuSuiteRun(suite);
     CuSuiteSummary(suite, &output);
